

/* TimeLapse
 * make a picture every day at time in table (TimeTable.h)
 * read RTC realtime clock chip DS1307
 * bring controller into sleep mode every 8 seconds to save batterie energy
 *
 * Arduino KHM board / ATmega168 or ATmega328
 * DS1307 Realtimeclock
 * 
 * 
 * KHM 1/2011 /  Martin Nawrath
 * Kunsthochschule fuer Medien Koeln
 * Academy of Media Arts Cologne
 */



#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <avr/wdt.h>

#include <WProgram.h>
#include <Wire.h>
#include <DS1307.h> // written by  mattt on the Arduino forum and modified by D. Sjunnesson
#include <SoftwareServo.h>  


SoftwareServo myservo1; 

#include "TimeTable.h"


char buff[50];
char st1[50];
int ibuf;

boolean f_receive=false;
boolean f_message=false;
boolean f_continu=false;

int cnt;
int ii;
int nint;
byte bb;

unsigned char phh,pmm,pss,pdd,pmo,pyy;
unsigned char rhh,rmm,rss,rdd,rmo,ryy,rdow,rssa;
long int pgm_sec,real_sec;

int pinLed=13;
int pinServo=11;
int pinRTC= 9;
int pinServoOn= 12;
int pinSpeaker = 8;

int numEntr=760;  // number of days/entrys in TimeTabe list


volatile boolean f_wdt;

void setup(){



  Serial.begin(115200);
  pinMode(pinLed,OUTPUT);
  pinMode(pinRTC,OUTPUT);
  pinMode(pinServo,OUTPUT);
  pinMode(pinSpeaker,OUTPUT);

  myservo1.attach(pinServo); 


  // CPU Sleep Mode Einstellung
  // SM2 SM1 SM0 Sleep Mode
  // 0    0  0 Idle
  // 0    0  1 ADC Noise Reduction
  // 0    1  0 Power-down
  // 0    1  1 Power-save
  // 1    0  0 Reserved
  // 1    0  1 Reserved
  // 1    1  0 Standby(1)
  // cbi( SMCR,SE );      // sleep enable

  SMCR &= ~(1<<SE);   // sleep enable
  SMCR &= ~(1<<SM0);   // power save mode
  SMCR |= (1<<SM1);   // power save mode
  SMCR &= ~(1<<SM2);   // sleep enable

  MCUSR &= ~(1<<WDRF);
  WDTCSR |= (1<<WDCE) | (1<<WDE); 


  WDTCSR = (1<<WDCE) | (1<<WDP0) | (1<<WDP3)   ; // WDT prescaler to 8sec

  WDTCSR |=  _BV(WDIE); // enable WDT interrupt

  digitalWrite(pinRTC,1);
  //  setRTC();

  printRTC();
  Serial.println("");
  Serial.println("Time Lapse Photo");

  beep();
  delay(500);
  beep();
  delay(500);
  beep();


  rtc_set_mode();  // ask for set the real time clock via terminal


  // print timetable to terminal
  for (int ii = 0; ii < numEntr; ii++) {
    phh= pgm_read_byte(&SRT[ii][0]); 
    pmm= pgm_read_byte(&SRT[ii][1]);  
    pss= pgm_read_byte(&SRT[ii][2]);  
    pdd= pgm_read_byte(&SRT[ii][3]);  
    pmo= pgm_read_byte(&SRT[ii][4]);  
    pyy= pgm_read_byte(&SRT[ii][5]);  
    sprintf(st1,"%4d %02d:%02d:%02d: %02d/%02d/%02d ",ii,phh,pmm,pss,pdd,pmo,pyy);
    Serial.println( st1 );
    delay(10);
    digitalWrite(pinSpeaker,!digitalRead(pinSpeaker));  
  }

  printRTC();
  Serial.println("");

  cameraSwitch();
  rssa=100;
  f_wdt=0;

}
//****************************************************************
//****************************************************************
//****************************************************************
void loop(){

  cnt++;
  delay(10);
  if (f_wdt==1) {
    f_wdt=0;
    Serial.print("Sleep cycle no:" );
    Serial.println(nint );
    delay(5);

    pinMode(pinLed,INPUT);
    pinMode(pinRTC,INPUT);
    pinMode(pinServo,INPUT);
    pinMode(pinServoOn,INPUT);
    pinMode(pinSpeaker,INPUT);
    //   cbi(ADCSRA,ADEN);
    ADCSRA &= ~(1<<ADEN); // disable ADC


    system_sleep();
    // wakeup now after 8 seconds
    // 10 ms awake for searching time
    // one minute for shoot 
    // Power; 150uA sleep / 20mA wake  on/off time ratio = 800


    pinMode(pinLed,OUTPUT);
    pinMode(pinRTC,OUTPUT);
    pinMode(pinServo,OUTPUT);
    pinMode(pinSpeaker,OUTPUT);
    digitalWrite(pinLed,1);
    digitalWrite(pinRTC,1);

    cnt=0;
    nint++;

    printRTC();

    // search through timetabbe
    for (int ii = 0; ii < numEntr; ii++) {
      phh= pgm_read_byte(&SRT[ii][0]); 
      pmm= pgm_read_byte(&SRT[ii][1]);  
      pss= pgm_read_byte(&SRT[ii][2]);  
      pdd= pgm_read_byte(&SRT[ii][3]);  
      pmo= pgm_read_byte(&SRT[ii][4]);  
      pyy= pgm_read_byte(&SRT[ii][5]); 
      // if date is right then look if also time correct
      if ( (pdd==rdd) & (pmo==rmo) & (pyy==ryy)) {

        sprintf(st1,"ShootTime at:  %02d:%02d:%02d: ",phh,pmm,pss);
        Serial.print(st1);
        pgm_sec=phh*60*60+pmm*60+pss;
        real_sec=rhh*60*60+rmm*60+rss;

        Serial.print("  minutes:");
        Serial.println((phh*60+pmm)-(rhh*60+rmm));
        // when seconds in range camera shoot
        if (  abs(pgm_sec- real_sec  ) < 15 ) {

          cameraSwitch();
          // 60 seconds pause
          for (int ww= 1 ; ww <=60; ww++) {
            delay(1000);
            beep();
            digitalWrite(pinLed,!digitalRead(pinLed));
            Serial.print(ww);
          }
          Serial.println("");
        }
      }
    }


    digitalWrite(pinRTC,0);
    digitalWrite(pinLed,0);
    if (rss == rssa) {
      Serial.println(" Warning: RTC off");
      for (int ii =0 ; ii< 5; ii++) {
        for (int tt=0; tt< 500; tt++) {
          delayMicroseconds(150);
          digitalWrite(pinSpeaker,!digitalRead(pinSpeaker));
        }

        delay(50);
      }
    }
    rssa=rss;
  }
}
//**************************************************************** 
void printRTC() {
  rhh=RTC.get(DS1307_HR,true);
  rmm=RTC.get(DS1307_MIN,true);
  rss=RTC.get(DS1307_SEC,true);
  rdd=RTC.get(DS1307_DATE,true);
  rmo=RTC.get(DS1307_MTH,true);
  ryy=RTC.get(DS1307_YR,true)-2000;
  rdow=RTC.get(DS1307_DOW,true);

  sprintf(st1,"clock %02d:%02d:%02d: %02d/%02d/%02d  DOW:%d",rhh,rmm,rss,rdd,rmo,ryy,rdow);

  Serial.println(st1);
}
//**************************************************************** 
void cameraSwitch() {

  int tt=0;
  int phi=0;

  pinMode(pinServoOn,OUTPUT); 
  digitalWrite(pinServoOn,1);
  Serial.println("shoot picture");

  for (phi=10 ; phi < 90; phi++){
    myservo1.write(phi);  
    delay(20);
    SoftwareServo::refresh();  
  }
  for (phi=90 ; phi > 10; phi--){
    myservo1.write(phi);  
    delay(20);
    SoftwareServo::refresh();  
  }

  delay(1000);
  digitalWrite(pinServoOn,0);
  digitalWrite(pinServoOn,0);

}

//****************************************************************  
void beep() {
  for (int tt=0; tt< 200; tt++) {
    delayMicroseconds(150);
    digitalWrite(pinSpeaker,!digitalRead(pinSpeaker));
  }
}
//****************************************************************  
void system_sleep() {
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);   // sleep mode is set here
  sleep_enable();
  sleep_mode(); 
  sleep_disable();


}
//****************************************************************  
// Watchdog interrupt service
ISR(WDT_vect) {
  f_wdt=1;

}

//**************************************************************** 
void setRTC() {

  pinMode(pinLed,OUTPUT);
  pinMode(pinRTC,OUTPUT);

  digitalWrite(pinLed,1);
  digitalWrite(pinRTC,1);

  RTC.stop();
  separateString(buff,st1,0);  
  rhh=atoi(st1);
  separateString(buff,st1,1);  
  rmm=atoi(st1);
  separateString(buff,st1,2);  
  rss=atoi(st1);
  separateString(buff,st1,3);  
  rdd=atoi(st1);
  separateString(buff,st1,4);  
  rmo=atoi(st1);
  separateString(buff,st1,5);  
  ryy=atoi(st1);
  separateString(buff,st1,6);  
  rdow=atoi(st1);


  sprintf(st1,"clock %02d:%02d:%02d: %02d/%02d/%02d  DOW:%d",rhh,rmm,rss,rdd,rmo,ryy,rdow);
  Serial.println(st1);



  RTC.set(DS1307_HR,rhh);      //set the hours
  RTC.set(DS1307_MIN,rmm);     //set the minutes
  RTC.set(DS1307_SEC,rss);     //set the seconds


    if (rdd !=0 ) {
    RTC.set(DS1307_DATE,rdd);   //set the date
    RTC.set(DS1307_MTH,rmo);     //set the month
    RTC.set(DS1307_YR,ryy);     //set the year
    RTC.set(DS1307_DOW,rdow);     //set the day of the week sun=1
  }
  RTC.start();

}

//*********************************************************************
void rtc_set_mode() {
  int cnt=0;
  int incnt=0;

  Serial.println("Set DS1307 RTC with '+++' command ");


  // wait for 3x '+' char receive
  while (cnt < 600 && incnt < 3) {
    delay(10);
    cnt++;
    if ( Serial.available()) { 
      if ('+' == Serial.read()) {
        cnt=0;
        incnt++;
        Serial.println(incnt);
      }  
    }
  }

  // clear serial buffer;
  while (Serial.available())   char inbyte =Serial.read() ;

  delay(10);
  if (incnt >= 3) {

    Serial.println("HH MM SS [Day Mon Year DOW] '.' '*' '\n' '\r' (sun=1 mo=2 tu=3 we=4 th=5 fr=6 sa=7)" );
    f_message=0;
    while (f_message ==0 && cnt < 1000) {  // 50 seconds timeout8 43 00 21 12 10 3
      delay(50);
      cnt++;
      receive_serial_message();   // Receive serial message via xbee

        if (f_message){     // if message complete
        Serial.println("Set RTC");
        setRTC();

        buff[0]='\0';          // clear message receive
      }
    }
  }



}

//*********************************************************************
void receive_serial_message() {

  if ( Serial.available() > 0) {     // if there are bytes waiting on the serial port
    f_receive = true;
    char inByte = Serial.read();   // read a byte
    buff[ibuf]=inByte;             // add to buffer  
    ibuf++;
    buff[ibuf]='\0';
    if (ibuf >= 50) ibuf=0;        // limit buffer size
    //  Serial.println(buff);
    if ((inByte == '.') ||(inByte == '*') || ( inByte == '\n') || ( inByte == '\r')) { // wait for message termination character
      //   separateString(buff,st1,2);       // separate message string into 3 substrings
      f_message=true;        // message avail now flag

      ibuf=0;
      f_receive = false;

      Serial.println(buff);
    }
  } 
}

//*********************************************************************
void separateString(char *bufin, char *bufout, int pos) {

  int cnt=0;
  char stc[]="0"; 
  bufout[0]='\0';


  for (int ii=0; ii<= strlen(bufin); ii++) {
    if ((bufin[ii] == ' ') &&  (bufin[ii+1] != ' ')) { //  Separator == all chars < ' '
      cnt++;
    } 
    else 
      stc[0] =  bufin[ii];

    if (cnt == pos  && bufin[ii] != ' ')  strcat(bufout, stc);
  }
}



















